//================================================================================================
// $Id: cgfile.h 6308 2023-01-16 20:16:41Z aron $
//
// Project:
//		Welltype Compiler
// Contents:
//		Code generator file helper
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTC_CGFILE_H
#define WTC_CGFILE_H

#include <string>
#include <vector>
#include "../wtruntime/wtruntime.h"

namespace wtc
{
class ProblemHandler;
} // wtc

namespace wtc
{
namespace codegen
{

class File;

class PostDataGenerator
{
protected:
	wtc::ProblemHandler & mProbHan;

public:
	PostDataGenerator(wtc::ProblemHandler & probhan) : mProbHan(probhan) { }
	virtual ~PostDataGenerator() { }
	virtual void Generate(std::vector<wt_u8> & out, const std::vector<wt_u8> & data) = 0;
};

class File
{
	struct PostData
	{
		PostData(wt_u32 offset, wt_u32 length, PostDataGenerator* generator)
			: offset(offset)
			, length(length)
			, generator(generator)
		{
			// ok
		}

		wt_u32 offset;
		wt_u32 length;
		PostDataGenerator* generator;
	};

	wtc::ProblemHandler & mProbHan;
	std::string           mFilename;
	bool                  mSwapByteorder;
	wt_u32                mHeaderOffset;
	std::vector<wt_u8>    mData;
	std::vector<PostData> mPostData;

	void PostProcess();

public:
	File(wtc::ProblemHandler & probhan, const std::string & filename,
		bool swap_byteorder);
	virtual ~File();

	wt_u32 GetSize() const { return (wt_u32)mData.size(); }

	void Write(const void* data, wt_u32 size);

	void WriteShebang(const std::string & s);

	void Write(wt_u8 v);
	void Write(wt_u16 v);
	void Write(wt_u32 v);
	void Write(wt_u64 v);

	void Write(const wtb_hdr & hdr);
	void Write(const wtb_shdr & shdr);
	void Write(const wtb_thdr & thdr);
	void Write(const wtb_metahdr & mhdr);
	void Write(const wtb_meta & meta);
	void Write(const wtb_symhdr & symhdr);
	void Write(const wtb_sym & sym);
	void Write(const wtb_dlabelhdr & dlabhdr);
	void Write(const wtb_dlabel & dlab);
	void Write(const wtb_integhdr & ihdr);
	void Write(const std::string & str_zero);

	void AddPostData(PostDataGenerator* gen, wt_u32 length);
};

} // codegen
} // wtc

#endif // WTC_CGFILE_H
