//================================================================================================
// $Id: validator.cpp 6308 2023-01-16 20:16:41Z aron $
//
// Project:
//		Welltype Compiler
// Contents:
//		Validator
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#include "validator.h"
#include "parserext.h"
#include <sstream>
#include "wtparser/astvisitor.h"
#include "wtparser/astnode.h"
#include "wtparser/checker.h"
#include "wtparser/ast/ast_functiondefinition.h"
#include "wtparser/ast/ast_functionsignature.h"

namespace wtc
{
namespace codegen
{
namespace validator
{

void ValidatePureProperty(wtc::ProblemHandler* probhan, wtast::FunctionDefinition* funcdef)
{
	const bool calc_pure = wtparser::checker::SubtreeIsPure(funcdef);
	const bool def_pure  = funcdef->GetFunctionSignature()->IsPure();

	if(calc_pure && !def_pure)
	{
		std::ostringstream ss;
		ss << "Function '";
		ss << funcdef->GetFunctionSignature()->GetCanonicalName(true, true);
		ss << "' defined as impure, but it is pure.";

		probhan->AddNote("?", 0, ss.str()); // TODO
	}
	else
	if(!calc_pure && def_pure)
	{
		std::ostringstream ss;
		ss << "Function '";
		ss << funcdef->GetFunctionSignature()->GetCanonicalName(true, true);
		ss << "' defined as pure, but it is impure.";

		probhan->AddError("?", 0, ss.str()); // TODO
	}
}

void ValidateReturnCoverage(wtc::ProblemHandler* probhan, wtast::FunctionDefinition* funcdef)
{
	wtast::FunctionSignature* sig = funcdef->GetFunctionSignature();

	if(0<sig->GetRetc())
	{
		if(!wtparser::checker::CheckFunctionReturnCoverage(funcdef))
		{
			std::ostringstream ss;
			ss << "Return statement missing in '";
			ss << funcdef->GetFunctionSignature()->GetCanonicalName(true, true);
			ss << "'.";

			probhan->AddError("?", 0, ss.str()); // TODO
		}
	}
}

} // validator
} // codegen
} // wtc
