#ifndef WTRUNTIME_OBJECTS_H
#define WTRUNTIME_OBJECTS_H

#include "config.h"
#include "types.h"
#include "debug.h"
#include "program.h"

#define WTR_OBJ_TO_VMADDR(_prog_,_obj_)   WTR_PTR_TO_VMADDR(_prog_,_obj_)
#define WTR_OBJ_FROM_VMADDR(_prog_,_vma_) WTR_PTR_FROM_VMADDR(_prog_,_vma_)

#ifdef ADD_INLINE_OBJECT_MAGIC
#  define WTR_MAGIC_STR1        0x57571010
#  define WTR_MAGIC_STR2        0xE5658356
#  define WTR_MAGIC_SEQ1        0x5E5E2020
#  define WTR_MAGIC_SEQ2        0xE5E56717
#  define WTR_MAGIC_REC1        0xECEC3030
#  define WTR_MAGIC_REC2        0xEEC61587
#endif /* ADD_INLINE_OBJECT_MAGIC */

struct wtr_program;

struct wtr_str
{
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic1;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u32 length;  /* length of the string */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic2;
	wt_u32 length_save;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	char   data[1]; /* content of the string (including the terminating zero) */
};

struct wtr_seq
{
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic1;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u32 length;   /* number of elements in the sequence */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic2;
	wt_u32 length_save;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u32 elemsize; /* size of an element in octets */
	wt_u8  data[];   /* data area */
};

struct wtr_rec
{
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic1;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u32 size;   /* size of the record in octets */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic2;
	wt_u32 size_save;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u8  data[]; /* data area */
};


/*----------------------------------------------------------------*/


/* String (or character sequence). Always contains an additional */
/* terminating zero. The 'data' parameter is optional, it can be */
/* NULL, but in this case  The string data will be uninitialized. */
struct wtr_str* wtr_str_create(struct wtr_program* prog, const char* data, wt_u32 len, wt_bool calc);
void wtr_str_delete(struct wtr_program* prog, struct wtr_str* str);


/* Sequence. An indexable sequence which contains elements. */
/* The element size cannot be zero. */
/* The virtual does not interpret the structures, this is only a shell. */
struct wtr_seq* wtr_seq_create(struct wtr_program* prog, wt_u32 elemsize, wt_u32 count);
void wtr_seq_delete(struct wtr_program* prog, struct wtr_seq* seq);

/* Record. A plain dynamically allocated memory area. */
/* The virtual does not interpret the structures, this is only a shell. */
struct wtr_rec* wtr_rec_create(struct wtr_program* prog, wt_u32 size);
void wtr_rec_delete(struct wtr_program* prog, struct wtr_rec* rec);


#ifdef ADD_INLINE_OBJECT_MAGIC

/* Determines whether the string object is valid or not. */
/* Returns: true, if magic1 and magic2 is valid */
/*   str - string object */
wt_bool wtr_str_is_valid(struct wtr_str* str);


/* Determines whether the sequence object is valid or not. */
/* Returns: true, if magic1 and magic2 is valid */
/*   seq - sequence object */
wt_bool wtr_seq_is_valid(struct wtr_seq* seq);


/* Determines whether the record object is valid or not. */
/* Returns: true, if magic1 and magic2 is valid */
/*   rec - record object */
wt_bool wtr_rec_is_valid(struct wtr_rec* rec);

#endif /* ADD_INLINE_OBJECT_MAGIC */

/*----------------------------------------------------------------*/

#endif /* WTRUNTIME_OBJECTS_H */
